@@ -1,12 +1,16 @@
 # -*- coding: utf-8 -*-
 
-__version__ = "0.0.22"
+__version__ = "0.0.33"
 __author__ = "BriFuture"
 
 """
-Update Since 0.0.22, socket server has been improved a lot, interface has changed
+0.0.33: move logger into _globals from utils to avoid log too much content
 
-Update Since 0.0.17: scripts in `myscripts` folder could add more scripts
+0.0.32: update with pyupdator
+
+0.0.22: socket server has been improved a lot, interface has changed
+
+0.0.17: scripts in `myscripts` folder could add more scripts
 without change main.py file.
 """
 
