@@ -1,8 +1,14 @@
 #!/usr/env/bin python3
 # -*- coding: utf-8 -*-
 
-"""Use as script entry
-"""
+'''
+ # @ Author: brifuture
+ # @ Create Time: 2021-04-02 15:33
+ # @ Modified by: brifuture
+ # @ Modified time: 2021-06-11 11:38
+ # @ Description: This File is created and the program is coded by BriFuture. (c) All rights reserved.
+    Use as script entry
+ '''
 
 import sys, os
 import os.path as osp
@@ -10,23 +16,28 @@ from ._constants import BFF_ROOT_PATH as RootPath
 
 from pathlib import Path
 from .utils import initGetText
+from ._globals import _logger as logger
 from . import __version__
 from ._plugin import load_scripts
 import json
 
 #  add custom sub-commands in this file
-with open(osp.join(RootPath, "myscripts/meta.json")) as f:
-    availableCmds = json.load(f)
-
+try:
+    with open(osp.join(RootPath, "myscripts/meta.json")) as f:
+        availableCmds = json.load(f)
+except:
+    availableCmds = {}
+    logger.warning(f"RootDir ({RootPath}) dose not contain meta " )
+    
 availableCmds["tray"] = "tray"
 
 tr = initGetText("bffacility")
 def onError(parser):
-    print(tr('Available sub commands: '))
+    logger.info(tr('Available sub commands: '))
     cmds = ""
     for i, c in enumerate(availableCmds):
         cmds += f"{i+1}: {c}  \t"
-    print(cmds, "\n")
+    logger.info(f"{cmds} \n")
     parser.print_usage()
 
 def loadPriScript(cmd, arg):
@@ -37,7 +48,7 @@ def loadPriScript(cmd, arg):
     pripath = Path(RootPath) / "../_pri"
     pripath = pripath.resolve()
     if len(arg) < 1:
-        print("Not Supported!")
+        logger.critical("Not Supported!")
         return
     try:
         subscript = arg[0]
@@ -45,13 +56,13 @@ def loadPriScript(cmd, arg):
         pripath = Path(osp.join(pripath, p))
         if ss.endswith(".py"):
             ss = ss[:-3]
-        print(ss, pripath)
+        logger.info(f"{ss}, {pripath}")
         arg = arg[1:]
         load_scripts(ss, arg, pripath)
     except Exception as e:
-        print("Running: ", e) 
+        logger.warning("Running: ", e) 
 
-def main():
+def main(**otherOpt):
     from argparse import ArgumentParser
     parser = ArgumentParser(prog='bffacility', 
         description="Usage: bffacility <subcommand> [args]")
@@ -63,6 +74,8 @@ def main():
     cmd = args["subcmd"]
     
     arg = sys.argv[1:]
+    
+    # logger.info(sys.stdout)
 
     if cmd == 'tray':
         from .win_tray import main as tray
@@ -72,7 +85,10 @@ def main():
         loadPriScript(cmd, arg)
         return
     elif cmd in availableCmds:
-        load_scripts(availableCmds[cmd], arg)
+        load_scripts(availableCmds[cmd], arg, **otherOpt)
         return
 
     onError(parser)
+
+# if __name__ == "__main__":
+#     main()
\ No newline at end of file
