@@ -1,10 +1,20 @@
+'''
+ # @ Author: brifuture
+ # @ Create Time: 2021-04-15 12:16
+ # @ Modified by: brifuture
+ # @ Modified time: 2021-06-11 12:01
+ # @ Description: This File is created and the program is coded by BriFuture. (c) All rights reserved.
+    Provides TCP/UDP socket Server and client
+ '''
+
 import json
 import socket
 import select
 from .utils import createLogger
 logger = createLogger('sockserver', savefile=False, stream=True)
+import traceback
 
-__version__ = "0.1.3"
+__version__ = "0.1.7"
 
 class EventHandler:
     def fileno(self):
@@ -39,38 +49,53 @@ class SocketClient(EventHandler):
         except_callback: function(client: SocketClient)
     """
     def __init__(self, sock, address, **kwargs):
-        self.sock = sock
-        self.sock.setblocking(False)
-        self.address = address
+        self._sock = sock
+        self._sock.setblocking(False)
+        if address is None:
+            try:
+                self.address = self._sock.getsockname()
+            except: 
+                self.address = None
+        else:
+            self.address = address
+
         self._isValid = True
 
         self._parse = None
         self._server = None
         self.except_callback = kwargs.get("except_callback", None)
+        self.bufferSize = kwargs.get("bufferSize", 2048)
+
     @staticmethod
     def createTcpClient(serverAddr):
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        sc = SocketClient(sock, ("127.0.0.1", 0))
-        sc.sock.setblocking(True)
+        sc = SocketClient(sock, None)
+        sc._sock.setblocking(True)
         sc._server = serverAddr
         return sc
 
+    @property
+    def sock(self):
+        return self._sock
+
     def connect(self):
         if self._server is None:
             return
         try:
-            self.sock.connect(self._server)
+            self._sock.connect(self._server)
+            if self.address is None:
+                self.address = self._sock.getsockname()
         except:
-            logger.warning(f"Error connecting {self._server}")
+            logger.warning(f"[SC] Error connecting {self._server} [{self.address}]")
             self.handle_except()
         finally:
             pass
 
     def shutdown(self):
-        self.sock.close()
+        self._sock.close()
 
     def __del__(self):
-        self.sock.close()
+        self._sock.close()
 
     @property
     def parser(self):
@@ -79,6 +104,7 @@ class SocketClient(EventHandler):
     @parser.setter
     def parser(self, parse):
         """传入一个解析器即可
+        @deprecated use setParser
         ```py
         # data is a dict
         def parse(data, socketClient): print(data, socketClient)
@@ -87,19 +113,29 @@ class SocketClient(EventHandler):
         ```
         """
         try:
-            parse('"test"', self)
+            parse('""', self) # pass empty string
             self._parse = parse
         except:
             logger.critical(f"[TC] ***** Error testing parser, it will be ignored ****")
 
+    def setParser(self, parser, testJson=True):
+        if testJson:
+            try:
+                parser('""', self) # pass empty string
+                self._parse = parser
+            except:
+                logger.critical(f"[TC] ***** Error testing parser, it will be ignored ****")
+        else:
+            self._parse = parser
+
     def is_valid(self):
         return self._isValid
 
     def fileno(self):
-        return self.sock.fileno()
+        return self._sock.fileno()
 
     def __repr__(self):
-        return f'<Socket Client, {self.sock.fileno()}, {self.address}>'
+        return f'<Socket Client, {self._sock.fileno()}, {self.address}>'
 
     def handle_receive(self):
         """Handle receive data, default processor
@@ -107,7 +143,7 @@ class SocketClient(EventHandler):
         """
         msg = None
         try:
-            msg = self.sock.recv(1024)
+            msg = self._sock.recv(self.bufferSize)
             if not msg:
                 self.handle_except()
                 return
@@ -122,7 +158,8 @@ class SocketClient(EventHandler):
             try:
                 self._parse(msg.decode(), self)
             except Exception as e:
-                logger.warning(f"[TC] parse {e}")
+                logger.warning(f"[TC] parse {msg}, {e}")
+                traceback.print_exc()
         else:
             logger.debug(msg)
 
@@ -131,7 +168,7 @@ class SocketClient(EventHandler):
     def handle_except(self):
         """注意不要在 callback 中重新创建 client，否则函数调用堆栈将会溢出
         """
-        self.sock.close()
+        self._sock.close()
         self._isValid = False
         if self.except_callback:
             self.except_callback(self)
@@ -156,6 +193,8 @@ class BaseServer():
             self.serve_once(0.1)
     
     def serve_once(self, timeout=0.001):
+        if len(self._recvList) == 0:
+            return
         can_recv, can_send, exc = select.select(self._recvList, self._sendList, self._exceptList, timeout)
         for h in can_recv:
             h.handle_receive()
@@ -175,6 +214,7 @@ class TcpSocketServer(EventHandler, BaseServer):
     def __init__(self, ip, port, maxconn = 500):
         super().__init__()
         self._server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self._server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         self._clients = {}
         # key: address tuple, value SocketClient
         self._recvList.append(self)
@@ -303,14 +343,20 @@ import threading
 class UdpSocketClient(EventHandler):
     def __init__(self, port, ip = "127.0.0.1", name="default", **kwargs):
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-        self.sock.bind((ip, port))
-        self.sock.setblocking(False)
         self.address = (ip, port)
+        self.sock.settimeout(0)
+        self.sock.setblocking(False)
+        self.sock.bind(self.address)
         self.bufferSize = kwargs.get("bufferSize", 1024)
-        self._isValid = True
         self.name = name
+        self._isValid = True
         self._parse = None
         self._peer = None
+        self._peers = []
+
+        self.except_callback = kwargs.get("except_callback", None)
+        # self.keepFirstPeer = False # if true, the first peer will always be used
+
     @property
     def parser(self):
         return self._parse
@@ -332,8 +378,8 @@ class UdpSocketClient(EventHandler):
         Args
             msg bytes
         """
-        if self._peer:
-            self.sock.sendto(msg, self._peer)
+        for p in self._peers:
+            self.sock.sendto(msg, p)
 
     def is_valid(self):
         return self._isValid
@@ -341,21 +387,35 @@ class UdpSocketClient(EventHandler):
     def fileno(self):
         return self.sock.fileno()
 
+    def __repr__(self):
+        return f"<UdpSocketClient {self.name} {self.address} {self.bufferSize} {self.sock.fileno()}>"
+
     def handle_receive(self):
         """Handle receive data, default processor
         subclass could replace this methods to receive binary data
         """
-        bytesAdressPair = self.sock.recvfrom(self.bufferSize)
-        # message = bytesAdressPair[0]
-        self._peer = bytesAdressPair[1]
-        if not bytesAdressPair:
-            self._isValid = False
+        bytesAddressPair = None
+        try:
+            bytesAddressPair = self.sock.recvfrom(self.bufferSize)
+        except Exception as e: 
+            # logger.warning(f"[UC] {self}  recv error : {e}")
+            # traceback.print_exc()
+            # ICMP recved cause the error, ignore it
+            pass
+
+        # message = bytesAddressPair[0]
+        if not bytesAddressPair:
+            # self.handle_except()
             return
+        if not bytesAddressPair[1] in self._peers:
+            logger.info(f"[UC] new client added: {bytesAddressPair[1]}")
+            self._peers.append(bytesAddressPair[1])
         if self._parse:
             try: 
-                self._parse(bytesAdressPair, self.name)
+                self._parse(bytesAddressPair, self.name)
             except Exception as e:
-                logger.warning(f"[UC] {e}")
+                logger.warning(f"[UC] parse error: {e}")
+                traceback.print_exc()
 
     def handle_send(self):
         pass
@@ -363,19 +423,53 @@ class UdpSocketClient(EventHandler):
     def handle_except(self):
         self.sock.close()
         self._isValid = False
+        if self.except_callback:
+            self.except_callback(self)
+
 
 class UdpSocketServer(BaseServer):
     """支持绑定到多个UDP端口上，使用 processor 进行回调
 
     Args:
         port udpsocket will bind into
+
+    Callback:
+        processor(msg: bytes, addr: tuple)
+
+    Usage:
+
+    ```py
+    server = UdpSocketServer(bufferSize = 4096)
+    
+    server.createListener(10000, "server1")
+    server.addProcessor(lambda x, y: print(x, y), name="server1")
+
+    # send 
+    server.sendto("server1", b"this is a test")
+    server.broadcast(b"this is a broadcast")
+
+    # in default loop
+    server.serve_forever()
+
+    # in custom loop
+    while True:
+        ## .. do other things
+        server.serve_once()
+        ## .. do other things
+    ```
+
     """
+    ClientClass = UdpSocketClient
     def __init__(self, **kwargs):
         super().__init__()
         self._listeners = {}
         self.bufferSize = kwargs.get("bufferSize", 1024)
         self._processors = {}
 
+    @property
+    def listeners(self):
+        return self._listeners
+
     def addProcessor(self, processor, name="default"):
         if name not in self._processors:
             self._processors[name] = processor
@@ -386,30 +480,52 @@ class UdpSocketServer(BaseServer):
     def createListener(self, port, name="default", ip="127.0.0.1", processor = None):
         if name in self._listeners.keys():
             return
+
+        assert port > 0 and port < 65535, "In valid Port"
+
         logger.info(f"[USS] Server Listening for '{name}': {ip}:{port}")
-        listener = UdpSocketClient(port, ip, bufferSize=self.bufferSize, name=name)
+        listener = self.ClientClass(port, ip, bufferSize=self.bufferSize, name=name)
         listener.parser = self.recvfrom
+        listener.except_callback = self.handle_except
+
         self._listeners[name] = listener
         self._recvList.append(listener)
         self._exceptList.append(listener)
         if processor is not None:
             self.addProcessor(processor, name)
         # self._sendList.append(listener)
+        return listener
+    
+    def handle_except(self, client):
+        logger.warning(f"[USS] Listener quit, {client.name}")
+        self._listeners.pop(client.name)
+        self._recvList.remove(client)
+        self._exceptList.remove(client)
 
-    def sendto(self, name, msg):
-        self._listeners[name].sendto(msg)
+    def send(self, msg: bytes, name = "default"):
+        """send msg to server by
+        """
+        if name not in self._listeners:
+            return
+        self._listeners[name].send(msg)
+
+    def sendto(self, msg: bytes, peer, name="default"):
+        if name not in self._listeners:
+            return
+        self._listeners[name].sock.sendto(msg, peer)
 
     def broadcast(self, msg):
         for c in self._listeners.values():
-            c.sendto(msg)
+            c.send(msg)
 
     def recvfrom(self, byteAddrPair, name):
-
+        """recv data from one udp server
+        """
         if name in self._processors:
             # process will executed
             process = self._processors[name]
             message = byteAddrPair[0]
             address = byteAddrPair[1]
-            process(message.decode(), address)
+            process(message, address)
         else:
             logger.debug(f"No processor for{name}, {byteAddrPair}")
