@@ -4,7 +4,10 @@ import os
 import atexit
 import logging
 from pathlib import Path
-_LOG_DIR = Path.home() / ".logs"   # Log Path
+
+__version__ = "0.1.1"
+
+_LOG_DIR = Path.home() / ".bfp" / ".logs"   # Log Path
 _LOG_DIR.mkdir(parents=True, exist_ok=True)
 
 formaterStr = "%(asctime)s %(levelname)s:  %(message)s"
@@ -21,6 +24,17 @@ def createLogger(name: str, savefile = True, stream = False,
     :: logger_prefix deprecated ::
 
     """
+
+    logger = logging.getLogger(name)
+    # print("create logger", name)
+    # import traceback
+    # traceback.print_stack()
+    if hasattr(logger, "_bf_inited"):
+        # return created logger
+        # print("Created Logger Returned")
+        return logger
+
+    logger._bf_inited = True
     if 'baseDir' in kwargs:
         # keep old compatbility
         basedir = kwargs["baseDir"]
@@ -32,12 +46,7 @@ def createLogger(name: str, savefile = True, stream = False,
     if not name.endswith(".log"):
         name = name + ".log"
     log_file = basedir / name
-
     _formater = logging.Formatter(formaterStr, kwargs.get("timeformat", "%Y-%m-%d %H:%M:%S"))
-    logger = logging.getLogger(name)
-    if len(logger.handlers) > 0:
-        # return created logger
-        return logger
 
     logger.setLevel(level)
     if savefile:
@@ -45,6 +54,7 @@ def createLogger(name: str, savefile = True, stream = False,
         fh.setFormatter(_formater)
         fh.setLevel(level)
         logger.addHandler(fh)
+        # print("add file handler", log_file)
     if stream:
         sh = logging.StreamHandler()
         sh.setLevel(level)
@@ -70,8 +80,6 @@ def initGetText(domain="myfacilities", dirs = _LANGUAGE_DIR) -> gettext.gettext:
     return gettext.gettext
 ## ********** For Translating **************
 
-_logger = createLogger('bffacilities')
-
 def lockfile(fileName, start = None, stop = None, logger=None):
     """
     original : Fix Multiple instances of scheduler problem  
@@ -82,8 +90,7 @@ def lockfile(fileName, start = None, stop = None, logger=None):
     :@Return True success
     """
     if logger is None:
-        global _logger
-        logger = _logger
+        logger = logging.getLogger("bffacilities")
     if platform.system() != 'Windows':
         fcntl = __import__("fcntl")
         f = open(fileName, "wb")
@@ -91,7 +98,7 @@ def lockfile(fileName, start = None, stop = None, logger=None):
             fcntl.flock(f, fcntl.LOCK_EX | fcntl.LOCK_NB)
             if start is not None:
                 start()
-            logger.debug("Scheduler Started...")
+            # logger.debug("Scheduler Started...")
         except Exception as e:
             logger.error('Exit the scheduler, error - {}'.format(e))
             if stop is not None:
@@ -109,7 +116,7 @@ def lockfile(fileName, start = None, stop = None, logger=None):
             msvcrt.locking(f.fileno(), msvcrt.LK_NBLCK, 1)
             if start is not None:
                 start()
-            logger.debug("Scheduler Started...")
+            # logger.debug("Scheduler Started...")
         except Exception as e:
             logger.error('Exit the scheduler, error - {}'.format(e))
             if stop is not None:
@@ -158,4 +165,5 @@ def createShortCut(target, link = None, desc="", workingdir=None, desktop=None,
         process = subprocess.Popen(script, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         # ret = p.stdout.read().decode()
     except Exception as e:
-        _logger.warn(f"Error when creating shortcut {e} for {script}")
+        logger = logging.getLogger("bffacilities")
+        logger.warn(f"Error when creating shortcut {e} for {script}")
